package com.taais.biz.service.impl;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import com.taais.biz.domain.Video2image;
import com.taais.biz.domain.bo.Video2imageBo;
import com.taais.biz.domain.vo.Video2imageVo;
import com.taais.biz.mapper.Video2imageMapper;
import com.taais.biz.service.IVideo2imageService;
import com.taais.biz.utils.VideoCapture;
import com.taais.common.core.utils.MapstructUtils;
import com.taais.common.orm.core.page.PageQuery;
import com.taais.common.core.core.page.PageResult;
import com.taais.common.orm.core.service.impl.BaseServiceImpl;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import static com.taais.biz.constant.BizConstant.VideoStatus.END;
import static com.taais.biz.constant.BizConstant.VideoStatus.RUNNING;
import static com.taais.biz.domain.table.Video2imageTableDef.VIDEO2IMAGE;

/**
 * 视频转图片 Service业务层处理
 *
 * @author km
 * 2024-08-16
 */
@Service
public class Video2imageServiceImpl extends BaseServiceImpl<Video2imageMapper, Video2image> implements IVideo2imageService {
    @Resource
    private Video2imageMapper video2imageMapper;

    @Override
    public QueryWrapper query() {
        return super.query().from(VIDEO2IMAGE);
    }

    private QueryWrapper buildQueryWrapper(Video2imageBo video2imageBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(VIDEO2IMAGE.NAME.like
            (video2imageBo.getName()));
        queryWrapper.and(VIDEO2IMAGE.STATUS.eq
            (video2imageBo.getStatus()));
        queryWrapper.and(VIDEO2IMAGE.OUT_PATH.eq
            (video2imageBo.getOutPath()));
        queryWrapper.and(VIDEO2IMAGE.START_TIME.eq
            (video2imageBo.getStartTime()));
        queryWrapper.and(VIDEO2IMAGE.END_TIME.eq
            (video2imageBo.getEndTime()));
        queryWrapper.and(VIDEO2IMAGE.COST_SECOND.eq
            (video2imageBo.getCostSecond()));
        queryWrapper.and(VIDEO2IMAGE.LOG.eq
            (video2imageBo.getLog()));
        queryWrapper.and(VIDEO2IMAGE.REMARKS.eq
            (video2imageBo.getRemarks()));
        queryWrapper.and(VIDEO2IMAGE.PATH.eq
            (video2imageBo.getPath()));
        queryWrapper.and(VIDEO2IMAGE.FPS.eq
            (video2imageBo.getFps()));

        return queryWrapper;
    }

    /**
     * 查询视频转图片
     *
     * @param id 视频转图片主键
     * @return 视频转图片
     */
    @Override
    public Video2imageVo selectById(Long id) {
        return this.getOneAs(query().where(VIDEO2IMAGE.ID.eq(id)), Video2imageVo.class);

    }

    /**
     * 查询视频转图片列表
     *
     * @param video2imageBo 视频转图片Bo
     * @return 视频转图片集合
     */
    @Override
    public List<Video2imageVo> selectList(Video2imageBo video2imageBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(video2imageBo);
        return this.listAs(queryWrapper, Video2imageVo.class);
    }

    /**
     * 分页查询视频转图片列表
     *
     * @param video2imageBo 视频转图片Bo
     * @return 分页视频转图片集合
     */
    @Override
    public PageResult<Video2imageVo> selectPage(Video2imageBo video2imageBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(video2imageBo);
        Page<Video2imageVo> page = this.pageAs(PageQuery.build(), queryWrapper, Video2imageVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增视频转图片
     *
     * @param video2imageBo 视频转图片Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(Video2imageBo video2imageBo) {
        Video2image video2image = MapstructUtils.convert(video2imageBo, Video2image.class);

        return this.save(video2image);// 使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 修改视频转图片
     *
     * @param video2imageBo 视频转图片Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(Video2imageBo video2imageBo) {
        Video2image video2image = MapstructUtils.convert(video2imageBo, Video2image.class);
        if (ObjectUtil.isNotNull(video2image) && ObjectUtil.isNotNull(video2image.getId())) {
            boolean updated = this.updateById(video2image);
            return updated;
        }
        return false;
    }

    /**
     * 批量删除视频转图片
     *
     * @param ids 需要删除的视频转图片主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }


    public void updateStatusToEnd(Long id, String log) {
        Video2image video2image = this.getById(id);
        video2image.setStatus(END);
        video2image.setLog(log);
        video2image.setEndTime(new Date());
        video2image.setCostSecond((video2image.getEndTime().getTime() - video2image.getStartTime().getTime()) / 1000);
        this.updateById(video2image);
    }
    @Override
    public boolean startToImage(Long id) {
        Video2image video2image = this.getById(id);
        if (ObjectUtil.isNull(video2image)) {
            System.out.println("ObjectUtil.isNotNull(video2image)...");
            return false;
        }
        System.out.println("开始截取图片...");

        String videoPath = video2image.getPath();
        String outPath = video2image.getOutPath();
        Long fps = video2image.getFps();

        System.out.println("videoPath: " + videoPath);
        System.out.println("outPath: " + outPath);
        System.out.println("fps: " + fps);

        video2image.setStatus(RUNNING);
        video2image.setStartTime(new Date());
        this.updateById(video2image);

        System.out.println("开始截取图片子线程...");

        new Thread(() -> {
            String log = "开始截取图片...\n";
            System.out.println("开始截取图片...");
            try {
                VideoCapture.startCaputre(videoPath, outPath, fps);
                System.out.println("视频转图片结束...");
                log += "视频转图片结束...\n";
            } catch (Exception e) {
                e.printStackTrace();
                log += "视频转图片出现位置错误...\n";
                log += e.getMessage();
            }
            updateStatusToEnd(id, log);
            System.out.println("视频转图片子线程结束...");
        }).start();

        return true;
    }
}
